#!/usr/bin/perl -w

package RPM::Source::Tools::SourceArchive;

use Cwd;
use Carp;
use strict;
use warnings;

use File::Temp qw/tempfile tempdir/;
use RPM::Source::Tools::SourceBundle;

our @ISA=qw/RPM::Source::Tools::SourceBundle/;
our $VERSION=0.001;

sub new {
    my $class = shift;
    my $self=$class->SUPER::new(@_);
    my (@unpackcmd, $listcmd);
    my $file = $self->{-file};
    if ($file=~/\.(?:tgz|tbz2?|txz|tar|tar\.(?:bz2?|gz|Z|xz))$/i) {
	@unpackcmd= ('tar','--overwrite','-x','-f');
	$listcmd= 'tar tf';
    } elsif ($file=~/\.zip$/i) {
	$listcmd= 'zipinfo -1';
	@unpackcmd= ('unzip');
    }
    $self->{-listcmd}=$listcmd;
    $self->{-unpackcmd}=\@unpackcmd;
    bless $self, $class;
    return $self;
}

sub extract_files {
    my ($self, $list)=@_;
    my $curdir = cwd();
    my $tmpdir = File::Temp->newdir('TEMPLATE' => 'archive-extract-XXXXXXXXXX');
    my $file=$self->{-file};
    chdir $tmpdir;
    # sometimes ./ is prepended to the path, as in GeoIP-1.4.8.tar.gz
    # in that case it should be restored with map {'./'.$_}, otherwise
    # tar: GeoIP-1.4.8/configure.in: not found in archive
    unless (system (@{$self->{-unpackcmd}}, $file, $self->{__filelist_leading_dot} ? map {'./'.$_} @$list : @$list)==0) {
	warn 'extracting '.join(' ',@$list)." from $file failed. Unpacking all...";
	system (@{$self->{-unpackcmd}}, $file)==0 or die "Unpacking $file failed.\n";
    }
    chdir $curdir;
    return $tmpdir;
}

sub filelist_cmd {
    my ($self,$file)=@_;
    my $listcmd=$self->{-listcmd};
    return unless $listcmd;
    return "$listcmd '$file'";
}

1;
